@@ -1,10 +1,11 @@
 package JavaScript::Duktape;
 use strict;
 use warnings;
+use Carp;
 use Data::Dumper;
 use Scalar::Util 'looks_like_number';
 
-our $VERSION = '0.0.1_1';
+our $VERSION = '0.0.1_2';
 
 use base qw/Exporter/;
 our @EXPORT = qw (
@@ -110,7 +111,6 @@ use constant {
     DUK_DEFPROP_HAVE_GETTER        => (1 << 7),
     DUK_DEFPROP_HAVE_SETTER        => (1 << 8),
     DUK_DEFPROP_FORCE              => (1 << 9),
-
     DUK_VARARGS                    => -1
 };
 
@@ -122,14 +122,7 @@ sub new {
     return $self;
 }
 
-my $t = 1;
-my $f = 0;
-my $n = '';
-my $null  = bless \$n, 'JS::NULL';
-my $true  = bless \$t, 'JS::TRUE';
-my $false = bless \$f, 'JS::FALSE';
-
-sub null  { $null }
+sub null  { $JavaScript::Duktape::Data::null; }
 sub true  { $JavaScript::Duktape::Bool::true; }
 sub false { $JavaScript::Duktape::Bool::false }
 
@@ -137,51 +130,73 @@ sub set {
     my $self = shift;
     my $name = shift;
     my $val = shift;
-    my $vm = $self->vm;
-    $vm->push_perl($val);
-    $vm->put_global_string($name);
+    my $duk = $self->vm;
+
+    if ($name =~ /\./){
+
+        my @props = split /\./, $name;
+        my $last = pop @props;
+        my $others = join '.', @props;
+
+        my $err = $duk->peval_string($others);
+        if ($err){ die $others . " is not a javascript object " }
+        
+        my $type = $duk->get_type(-1);
+        if ($type != DUK_TYPE_OBJECT){
+            die $others . " isn't an object";
+        }
+
+        $duk->push_string($last);
+        $duk->push_perl($val);
+        $duk->put_prop(-3);
+        $duk->pop();
+        return 1;
+    }
+
+    $duk->push_perl($val);
+    $duk->put_global_string($name);
     return 1;
 }
 
 sub get {
     my $self = shift;
     my $name = shift;
-    my $vm = $self->vm;
-    $vm->push_string($name);
-    $vm->eval();
-    my $ret = $vm->to_perl(-1);
-    $vm->pop();
+    my $duk = $self->vm;
+    $duk->push_string($name);
+    my $err = $duk->peval();
+    if ($err){
+        my $error = $duk->safe_to_string(-1);
+        croak $error;
+    }
+
+    my $ret = $duk->to_perl(-1);
+    $duk->pop();
     return $ret;
 }
 
+sub eval {
+    my $self = shift;
+    my $string = shift;
+    my $duk = $self->duk;
+
+    my $err = $duk->peval_string($string);
+    
+    if ($err){
+        my $error_string = $duk->safe_to_string(-1);
+        croak $error_string;
+    }
+
+    return $duk->to_perl(-1);
+}
+
 sub vm { shift->{duk}; }
 sub duk { shift->{duk}; }
 
-=head1 NAME
-
-JavaScript::Duktape - Perl interface to Duktape embeddable javascript engine
-
-=head1 SYNOPSIS
-    
-    use JavaScript::Duktape;
-
-    ##create new js context
-    my $js = JavaScript::Duktape->new();
-
-    #set function to be used from javascript land
-    $js->set('write' => sub {
-        print $_[0], "\n";
-    });
-
-    $js->eval(qq~
-        (function(){
-            for (var i = 0; i < 100; i++){
-                write(i);
-            }
-        })();
-    ~);
-=cut
-
+sub DESTROY {
+    my $self = shift;
+    my $duk = $self->duk;
+    $duk->destroy_heap();
+}
 
 package JavaScript::Duktape::Vm;
 use strict;
@@ -207,6 +222,7 @@ use Inline C => config =>
     typemaps => JavaScript::Duktape::C::libPath::getPath('typemap'),
     INC => '-I' . JavaScript::Duktape::C::libPath::getPath('../C');
 
+#XXX : build duktape as a shared library and link it
 # use Inline C => config =>
 #     myextlib => $Duklib,
 #     LIBS => '-L'. JavaScript::Duktape::C::libPath::getPath('../C') . ' -lduktape';
@@ -216,66 +232,13 @@ use Inline C => JavaScript::Duktape::C::libPath::getPath('duktape_wrap.c');
 my $SUB = 0;
 my $Functions = {};
 
-use constant {
-    DUK_TYPE_NONE      => 0,
-    DUK_TYPE_UNDEFINED => 1,
-    DUK_TYPE_NULL      => 2,
-    DUK_TYPE_BOOLEAN   => 3,
-    DUK_TYPE_NUMBER    => 4,
-    DUK_TYPE_STRING    => 5,
-    DUK_TYPE_OBJECT    => 6,
-    DUK_TYPE_BUFFER    => 7,
-    DUK_TYPE_POINTER   => 8,
-    DUK_TYPE_LIGHTFUNC => 9,
-
-    DUK_TYPE_MASK_NONE       => (1 << 0),
-    DUK_TYPE_MASK_UNDEFINED  => (1 << 1),
-    DUK_TYPE_MASK_NULL       => (1 << 2),
-    DUK_TYPE_MASK_BOOLEAN    => (1 << 3),
-    DUK_TYPE_MASK_NUMBER     => (1 << 4),
-    DUK_TYPE_MASK_STRING     => (1 << 5),
-    DUK_TYPE_MASK_OBJECT     => (1 << 6),
-    DUK_TYPE_MASK_BUFFER     => (1 << 7),
-    DUK_TYPE_MASK_POINTER    => (1 << 8),
-    DUK_TYPE_MASK_LIGHTFUNC  => (1 << 9),
-    DUK_TYPE_MASK_THROW      => (1 << 10),
-
-    # Enumeration flags for duk_enum()
-    DUK_ENUM_INCLUDE_NONENUMERABLE => (1 << 0),
-    DUK_ENUM_INCLUDE_INTERNAL      => (1 << 1),
-    DUK_ENUM_OWN_PROPERTIES_ONLY   => (1 << 2),
-    DUK_ENUM_ARRAY_INDICES_ONLY    => (1 << 3),
-    DUK_ENUM_SORT_ARRAY_INDICES    => (1 << 4),
-    DUK_ENUM_NO_PROXY_BEHAVIOR     => (1 << 5),
-
-    DUK_COMPILE_EVAL               => (1 << 0),   
-    DUK_COMPILE_FUNCTION           => (1 << 1),
-    DUK_COMPILE_STRICT             => (1 << 2),
-    DUK_COMPILE_SAFE               => (1 << 3),
-    DUK_COMPILE_NORESULT           => (1 << 4),
-    DUK_COMPILE_NOSOURCE           => (1 << 5),
-    DUK_COMPILE_STRLEN             => (1 << 6),
-
-    #Flags for duk_def_prop() and its variants
-    DUK_DEFPROP_WRITABLE           => (1 << 0),
-    DUK_DEFPROP_ENUMERABLE         => (1 << 1),
-    DUK_DEFPROP_CONFIGURABLE       => (1 << 2),
-    DUK_DEFPROP_HAVE_WRITABLE      => (1 << 3),
-    DUK_DEFPROP_HAVE_ENUMERABLE    => (1 << 4),
-    DUK_DEFPROP_HAVE_CONFIGURABLE  => (1 << 5),
-    DUK_DEFPROP_HAVE_VALUE         => (1 << 6),
-    DUK_DEFPROP_HAVE_GETTER        => (1 << 7),
-    DUK_DEFPROP_HAVE_SETTER        => (1 << 8),
-    DUK_DEFPROP_FORCE              => (1 << 9),
-};
-
 sub push_perl {
     my $self = shift;
     my $val = shift;
 
     my $ref = ref $val;
     if ($ref){
-        if ($ref eq 'JS::NULL'){
+        if ($ref eq 'JavaScript::Duktape::Data'){
             $self->push_null();
         } elsif ($ref eq 'JavaScript::Duktape::Bool'){
             if ($val){
@@ -327,15 +290,15 @@ sub to_perl {
     
     my $type = $self->get_type($index);
 
-    if ($type == DUK_TYPE_STRING){
+    if ($type == JavaScript::Duktape::DUK_TYPE_STRING){
         $ret = $self->get_string($index);
     }
 
-    elsif ($type == DUK_TYPE_NUMBER){
+    elsif ($type == JavaScript::Duktape::DUK_TYPE_NUMBER){
         $ret = $self->get_number($index) + 0;
     }
 
-    elsif ($type == DUK_TYPE_OBJECT){
+    elsif ($type == JavaScript::Duktape::DUK_TYPE_OBJECT){
 
         if ($self->is_function($index)){
             my $ptr = $self->get_heapptr($index);
@@ -354,7 +317,7 @@ sub to_perl {
 
         my $isArray = $self->is_array($index);
         $ret = $isArray ? [] : {};
-        $self->enum($index, DUK_ENUM_OWN_PROPERTIES_ONLY);
+        $self->enum($index, JavaScript::Duktape::DUK_ENUM_OWN_PROPERTIES_ONLY);
 
         while ($self->next(-1, 1)) {
             my $key = $self->to_perl(-2);
@@ -371,7 +334,7 @@ sub to_perl {
         $self->pop();
     }
 
-    elsif ($type == DUK_TYPE_BOOLEAN){
+    elsif ($type == JavaScript::Duktape::DUK_TYPE_BOOLEAN){
         my $bool =  $self->get_boolean($index);
         if ($bool == 1){
             $ret = JavaScript::Duktape::Bool::true();
@@ -380,11 +343,12 @@ sub to_perl {
         }
     }
 
-    elsif ($type == DUK_TYPE_NULL){
-        $ret = $null;
+    elsif ($type == JavaScript::Duktape::DUK_TYPE_NULL){
+        $ret = JavaScript::Duktape::Data::null();
     }
 
-    elsif ($type == DUK_TYPE_POINTER){
+    ##XXX to do
+    elsif ($type == JavaScript::Duktape::DUK_TYPE_POINTER){
         $ret = "pointer";
     }
 
@@ -395,6 +359,194 @@ sub to_perl {
     return $ret;
 }
 
+##############################################
+# push functions
+##############################################
+sub push_function {
+    my $self = shift;
+    my $sub = shift;
+    my $nargs = shift;
+    if (!defined $nargs){ $nargs = -1 }
+
+    $SUB++;
+    $Functions->{$SUB} = sub {
+        my $top = $self->get_top();
+        my $ret = 1;
+        $self->perl_duk_safe_call(sub {
+            eval { $ret = $sub->($self) };
+            my $error = $@;
+            if ($error){
+                if ($error =~ /^Duk::Error/){
+                    die $@;
+                } else {
+                    $error =~ s/\n//g;
+                    $error =~ s/\\/\\\\/g;
+                    $self->eval_string("throw new Error('$error')");
+                }
+            }
+            return $ret;
+        }, $top, 1);
+        return $ret;
+    };
+
+    $self->perl_push_function($Functions->{$SUB}, $nargs);
+}
+
+*push_c_function = \&push_function;
+
+#####################################################################
+# safe call
+#####################################################################
+sub safe_call {
+    my $self = shift;
+    my $sub = shift;
+    my $safe = sub {
+        return $sub->($self);
+    };
+
+    my $oldtop = $self->get_top();
+    eval { $self->perl_duk_safe_call($safe, @_) };
+    my $error = $@;
+    if ($error) {
+        if ($error =~ /^Duk::Error/i) {
+            # error came from duktape do nothing;
+        } else {
+            #error from perl push error object
+            #and clear previous pushed stack
+            my $newtop = $self->get_top();
+            if ($newtop > $oldtop){
+                $self->pop_n($newtop - $oldtop);
+            }
+            $error =~ s/\n//g;
+            $error =~ s/\\/\\\\/g;
+            $self->reset_top();
+            $self->eval_string("var t = new Error('$error'); t;");
+        }
+        return 1;
+    }
+    return 0;
+}
+
+##############################################
+# overridden functions
+##############################################
+*require_context = \&perl_duk_require_context;
+
+
+##############################################
+# helper functions
+##############################################
+*reset_top = \&perl_duk_reset_top;
+
+sub dump {
+    my $self = shift;
+    my $name = shift || "Duktape";
+    my $fh  = shift || \*STDOUT;
+    my $n = $self->get_top();
+    printf $fh "%s (top=%ld):", $name, $n;
+    for (my $i = 0; $i < $n; $i++) {
+        printf $fh " ";
+        $self->dup($i);
+        printf $fh "%s", $self->safe_to_string(-1);
+        $self->pop();
+    }
+    printf $fh "\n";
+}
+
+sub DESTROY {}
+
+package JavaScript::Duktape::Bool; {
+    use warnings;
+    use strict;
+    our ($true, $false);
+    use overload
+        '""' => sub { ${$_[0]} },
+        'bool' => sub { ${$_[0]} ? 1 : 0 },
+        fallback => 1;
+
+    BEGIN {
+        my $t = 1;
+        my $f = 0;
+        $true  = bless \$t, 'JavaScript::Duktape::Bool';
+        $false = bless \$f, 'JavaScript::Duktape::Bool';
+    }
+
+    sub true  { $true }
+    sub false { $false }
+}
+
+package JavaScript::Duktape::Data; {
+    use warnings;
+    use strict;
+    our ($null);
+    use overload
+        '""' => sub { ${$_[0]} },
+        'bool' => sub { ${$_[0]} ? 1 : 0 },
+        fallback => 1;
+
+    BEGIN {
+        my $n = '';
+        $null  = bless \$n, 'JavaScript::Duktape::Data';
+    }
+
+    sub null  { $null }
+}
+
+1;
+
+__END__
+=head1 NAME
+
+JavaScript::Duktape - Perl interface to Duktape embeddable javascript engine
+
+=for html
+<a href="https://travis-ci.org/mamod/JavaScript-Duktape"><img src="https://travis-ci.org/mamod/JavaScript-Duktape.svg?branch=master"></a>
+
+=head1 SYNOPSIS
+    
+    use JavaScript::Duktape;
+
+    ##create new js context
+    my $js = JavaScript::Duktape->new();
+
+    #set function to be used from javascript land
+    $js->set('write' => sub {
+        my $duk = shift;
+        print $_[0], "\n";
+    });
+
+    $js->eval(qq~
+        (function(){
+            for (var i = 0; i < 100; i++){
+                write(i);
+            }
+        })();
+    ~);
+
+=head1 DESCRIPTION
+
+JavaScript::Duktape implements almost all duktape javascript engine api, the c code is just 
+a thin layer that maps duktape api to perl, and all other functions implemented in perl
+it self, so maintaing and contributing to the base code should be easy.
+
+=head1 methods
+
+=over 4
+
+=item set('name', data);
+
+Creates properity 'name' and sets it's value to the given perl data
+
+=item get('name');
+
+Gets properity 'name' value from javascript and return it as perl data
+
+=item eval('javascript');
+
+Evaluates javascript string and return the results or croak if error
+
+=back
+
 =head1 VM API
 
 vm api corresponds to Duktape Engine API see L<http://duktape.org/api.html>
@@ -442,114 +594,68 @@ above but with using C<dump> function to get a glance of stack top
     #Bingo
     $duk->dump(); #-> [ Duktape (top=0): ]
 
-=cut
+=head1 VM methods
 
-##############################################
-# eval functions
-##############################################
+As a general rule all duktape api supported, but I haven't had the chance to test them all, 
+so please report any missing or failure api call and I'll try to fix
 
-sub call_function {
-    my $args = shift;
-    my $self = $args->{self};
-    my $sub = $args->{sub};
-    my $top = $self->get_top();
-    eval { $self->perl_duk_safe_call($sub, $top, 1) };
-    if ($@){ die $@; }
-    return 0;
-}
+For the list of duktape engine API please see L<http://duktape.org/api.html>, and here is how
+you can translate duktape api to perl
 
-sub push_function {
-    my $self = shift;
-    my $sub = shift;
-    my $nargs = shift;
-    if (!defined $nargs){ $nargs = -1 }
+    my $js = JavaScript::Duktape->new();
+    my $duk = $js->duk;
 
-    $SUB++;
-    $Functions->{$SUB} = {
-        sub => sub {
-            my $ret = $sub->($self);
-            if ($ret > 1){
-                $self->eval_string("throw new Error('unknown return value')");
-                return 1;
-            }
-            return $ret;
-        },
-        self => $self
-    };
-    $self->perl_push_function($Functions->{$SUB}, $nargs);
-}
+    # -- C example
+    # duk_push_c_function(func, 2);
+    # duk_push_int(ctx, 2);
+    # duk_push_int(ctx, 3);
+    # duk_call(ctx, 2);  /* [ ... func 2 3 ] -> [ 5 ] */
+    # printf("2+3=%ld\n", (long) duk_get_int(ctx, -1));
+    # duk_pop(ctx);
 
-#####################################################################
-# safe call
-#####################################################################
-sub safe_call {
-    my $self = shift;
-    my $sub = shift;
-    my $safe = sub {
-        my $ret;
-        eval { $ret = $sub->($self) };
-        my $error = $@;
-        if ($error) {
-            if ($error =~ /^Duk::Error/i) {
-                die;
-            } else {
-                $error =~ s/\n//g;
-                $self->peval_string_noresult("throw new Error('$error')");
-            }
-        }
-        return $ret;
-    };
+    #and here is how we can implement it in JavaScript::Duktape
+    
+    $duk->push_c_function(sub { 
+        my $duk = shift;
+        my $num1 = shift;
+        my $num2 = shift;
 
-    eval { $self->perl_duk_safe_call($safe, @_) };
-    if ($@){ return 1 }
-    return 0;
-}
+        my $total = $num1+$num2;
+        $duk->push_number($total);
+        return 1;
+    }, 2);
 
-##############################################
-# overridden functions
-##############################################
-*require_context = \&perl_duk_require_context;
+    $duk->push_int(2);
+    $duk->push_int(3);
+    $duk->call(2);  # [ ... func 2 3 ] -> [ 5 ]
+    printf("2+3=%ld\n", $duk->get_int(-1));
+    $duk->pop();
 
+As you can see all you need to do is replacing C<duk_> with C<$duk->> and remove C<ctx> from the function call,
+this may sounds crazy but api tests have been generated by copying duktape tests and using search and replace tool :)
 
-##############################################
-# helper functions
-##############################################
-*reset_top = \&perl_duk_reset_top;
+Besides duktape api, C<JavaScript::Duktape::Vm> implements the following methods 
 
-sub dump {
-    my $self = shift;
-    my $name = shift || "Duktape";
-    my $fh  = shift || \*STDOUT;
-    my $n = $self->get_top();
-    printf $fh "%s (top=%ld):", $name, $n;
-    for (my $i = 0; $i < $n; $i++) {
-        printf $fh " ";
-        $self->dup($i);
-        printf $fh "%s", $self->safe_to_string(-1);
-        $self->pop();
-    }
-    printf $fh "\n";
-}
+=over 4
+ 
+=item push_function ( code_ref, num_of_args );
+ 
+an alias to push_c_function
 
-package JavaScript::Duktape::Bool; {
-    use warnings;
-    use strict;
-    our ($true, $false);
-    use overload
-        '""' => sub { ${$_[0]} },
-        # '!' => sub { !${$_[0]} },
-        'bool' => sub { ${$_[0]} ? 1 : 0 },
-        fallback => 1;
+=item push_perl( ... );
 
-    BEGIN {
-        my $t = 1;
-        my $f = 0;
-        $true  = bless \$t, 'JavaScript::Duktape::Bool';
-        $false = bless \$f, 'JavaScript::Duktape::Bool';
-    }
+Push given perl data into the duktape stack.
 
-    sub true  { $true }
-    sub false { $false }
-}
+=item to_perl(index);
 
-1;
+Get the value at index and return it as perl data
+
+=item reset_top
+
+resets duktape stack top
+
+=back
+
+=head1 AUTHOR
+ 
+Mamod Mehyar C<< <mamod.mehyar@gmail.com> >>
